==> org-lsst-ccs-ui-2.4.11-sources.jar: org/lsst/ccs/gconsole/agent/command/CommandSender.java <==
package org.lsst.ccs.gconsole.agent.command;

import java.time.Duration;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.lsst.ccs.Agent;
import org.lsst.ccs.messaging.AgentMessagingLayer;
import org.lsst.ccs.utilities.scheduler.Scheduler;

/**
 * Utility class for sending commands to remote subsystems and processing responses.
 * The sender can be configured by calling its setter methods. 
 * This class is thread-safe, all methods can be called on any thread.
 *
 * @author onoprien
 */
public class CommandSender {

// -- Fields : -----------------------------------------------------------------
    
    final AgentMessagingLayer messenger;
    final Executor callbackExecutor;
    final Executor workerExecutor;
    final Scheduler timer;
            
    private volatile Duration defaultTimeout = Duration.ofSeconds(10);
    private volatile CommandHandle defaultCommandHandle;

// -- Life cycle : -------------------------------------------------------------
    
    /**
     * Creates an instance of {@code CommandSender}.
     * 
     * @param messenger Messaging access point. If {@code null}, the environment messaging access will be used.
     * @param callbackExecutor Executor for running {@link CommandHandle} methods. If {@code null}, a dedicated single-threaded executor will be created.
     * @param workerExecutor Executor for running potentially time-consuming tasks. If {@code null}, a dedicated executor will be created.
     * @param timer Scheduling executor for timing commands.
     */
    public CommandSender(AgentMessagingLayer messenger, Executor callbackExecutor, Executor workerExecutor, Scheduler timer) {
        
        if (messenger == null) {
            messenger = Agent.getEnvironmentMessagingAccess();
        }
        this.messenger = messenger;
        
        if (callbackExecutor == null || workerExecutor == null) {
            ThreadFactory threadFactory = new ThreadFactory() {
                private final ThreadFactory delegate = Executors.defaultThreadFactory();
                @Override
                public Thread newThread(Runnable r) {
                    Thread thread = delegate.newThread(r);
                    thread.setDaemon(true);
                    return thread;
                }
                
            };
            if (callbackExecutor == null) {
                callbackExecutor = Executors.newSingleThreadExecutor(threadFactory);
            }
            if (workerExecutor == null) {
                workerExecutor = new ThreadPoolExecutor(0, 2, 60L, TimeUnit.SECONDS, new SynchronousQueue<>(), threadFactory);
            }
        }
        this.callbackExecutor = callbackExecutor;
        this.workerExecutor = workerExecutor;
        this.timer = timer;
    }
    
    
// -- Setters : ----------------------------------------------------------------
    
    /**
     * Sets the default command timeout for this sender.
     * The specified timeout will be used for command execution requests that do not provide their own timeout values.
     * If this method has not been called, the default timeout value is 10 seconds.
     * 
     * @param defaultTimeout Default timeout. Zero means the command will never time out.
     *                Negative timeout means its absolute value will be used and the custom timeout
     *                suggested by the target subsystem through an ACK will be ignored.
     */
    public void setTimeout(Duration defaultTimeout) {
        if (defaultTimeout == null) {
            throw new IllegalArgumentException("Illegal default timeout value: "+ defaultTimeout);
        }
        this.defaultTimeout = defaultTimeout;
    }
    
    /**
     * Sets the default command handle for this sender.
     * The specified handle will be used for command execution requests that do not provide their own handle.
     * 
     * @param defaultCommandHandle Default command handle.
     */
    public void setCommandHandle(CommandHandle defaultCommandHandle) {
        this.defaultCommandHandle = defaultCommandHandle;
    }
    
// -- Sending commands : -------------------------------------------------------
    
    /**
     * Sends a command to a remote subsystem.
     * This method returns immediately, without waiting for the command to be processed
     * by the messaging service or the target subsystem. Synchronous command execution can
     * be done by calling {@link CommandTask#get get()} or {@link CommandTask#getResult() getResult()}
     * on the {@link CommandTask} returned by this method.
     * 
     * @param handle Handle for processing the command responses. 
     *               If {@code null}, the default command handle of this sender will be used if it has been set,
     *               or no processing will be performed if it has not been set..
     * @param timeout Timeout. If {@code null}, the default timeout of this sender will be used.
     *                Zero means the command will never time out.
     *                Negative timeout means its absolute value will be used and the custom timeout
     *                suggested by the target subsystem through an ACK will be ignored.
     * @param command Command in subsystem/path format.
     * @param args Command arguments.
     * @return CommandTask created to process the command. Can be used to wait for the response or cancel the command processing.
     */
    public CommandTask execute(CommandHandle handle, Duration timeout, String command, Object... args) {
        if (handle == null) handle = defaultCommandHandle;
        if (timeout == null) timeout = defaultTimeout;
        CommandTask task = new CommandTask(this, handle, timeout, command, args);
        task.send();
        return task;
    }

    
}

==> org-lsst-ccs-ui-3.1.1-sources.jar: org/lsst/ccs/gconsole/agent/command/CommandSender.java <==
package org.lsst.ccs.gconsole.agent.command;

import java.time.Duration;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.lsst.ccs.Agent;
import org.lsst.ccs.messaging.AgentMessagingLayer;
import org.lsst.ccs.utilities.scheduler.Scheduler;

/**
 * Utility class for sending commands to remote subsystems and processing responses.
 * The sender can be configured by calling its setter methods. 
 * This class is thread-safe, all methods can be called on any thread.
 *
 * @author onoprien
 */
public class CommandSender {

// -- Fields : -----------------------------------------------------------------
    
    final AgentMessagingLayer messenger;
    final Executor callbackExecutor;
    final Executor workerExecutor;
    final Scheduler timer;
            
    private volatile Duration defaultTimeout = Duration.ofSeconds(10);
    private volatile CommandHandle defaultCommandHandle;

// -- Life cycle : -------------------------------------------------------------
    
    /**
     * Creates an instance of {@code CommandSender}.
     * 
     * @param messenger Messaging access point. If {@code null}, the environment messaging access will be used.
     * @param callbackExecutor Executor for running {@link CommandHandle} methods. If {@code null}, a dedicated single-threaded executor will be created.
     * @param workerExecutor Executor for running potentially time-consuming tasks. If {@code null}, a dedicated executor will be created.
     * @param timer Scheduling executor for timing commands.
     */
    public CommandSender(AgentMessagingLayer messenger, Executor callbackExecutor, Executor workerExecutor, Scheduler timer) {
        
        if (messenger == null) {
            messenger = Agent.getEnvironmentMessagingAccess();
        }
        this.messenger = messenger;
        
        if (callbackExecutor == null || workerExecutor == null) {
            ThreadFactory threadFactory = new ThreadFactory() {
                private final ThreadFactory delegate = Executors.defaultThreadFactory();
                @Override
                public Thread newThread(Runnable r) {
                    Thread thread = delegate.newThread(r);
                    thread.setDaemon(true);
                    return thread;
                }
                
            };
            if (callbackExecutor == null) {
                callbackExecutor = Executors.newSingleThreadExecutor(threadFactory);
            }
            if (workerExecutor == null) {
                workerExecutor = new ThreadPoolExecutor(0, 2, 60L, TimeUnit.SECONDS, new SynchronousQueue<>(), threadFactory);
            }
        }
        this.callbackExecutor = callbackExecutor;
        this.workerExecutor = workerExecutor;
        this.timer = timer;
    }
    
    
// -- Setters : ----------------------------------------------------------------
    
    /**
     * Sets the default command timeout for this sender.
     * The specified timeout will be used for command execution requests that do not provide their own timeout values.
     * If this method has never been called, the default timeout value is 10 seconds.
     * 
     * @param defaultTimeout Default timeout.
     *                Zero duration means the command will never time out.
     *                Negative duration means its absolute value will be used and the custom timeout
     *                suggested by the target subsystem through an ACK will be ignored.
     * @throws IllegalArgumentException If the supplied value is {@code null}.
     */
    public void setTimeout(Duration defaultTimeout) {
        if (defaultTimeout == null) {
            throw new IllegalArgumentException("Illegal default timeout value: "+ defaultTimeout);
        }
        this.defaultTimeout = defaultTimeout;
    }
    
    /**
     * Sets the default command handle for this sender.
     * The specified handle will be used for command execution requests that do not provide their own handle.
     * If this method has never been called, the default handle does nothing.
     * 
     * @param defaultCommandHandle Default command handle.
     */
    public void setCommandHandle(CommandHandle defaultCommandHandle) {
        this.defaultCommandHandle = defaultCommandHandle;
    }
    
// -- Sending commands : -------------------------------------------------------
    
    /**
     * Sends a command to a remote subsystem.
     * <p>
     * This method returns immediately, without waiting for the command to be processed
     * by the messaging service or the target subsystem. Synchronous command execution can
     * be done by calling {@link CommandTask#get get()} or {@link CommandTask#getResult() getResult()}
     * on the {@link CommandTask} returned by this method.
     * 
     * @param handle Handle for processing the command responses. May be {@code null} if no processing is required.
     * @param timeout Timeout. Cannot be {@code null}.
     *                Zero duration means the command will never time out.
     *                Negative duration means its absolute value will be used and the custom timeout
     *                suggested by the target subsystem through an ACK will be ignored.
     * @param command Command in subsystem/path format.
     * @param args Command arguments.
     * @return CommandTask created to process the command. Can be used to wait for the response or cancel the command processing.
     */
    public CommandTask execute(CommandHandle handle, Duration timeout, String command, Object... args) {
        if (handle == null) handle = defaultCommandHandle;
        CommandTask task = new CommandTask(this, handle, timeout, command, args);
        task.send();
        return task;
    }
    
    /**
     * Sends a command to a remote subsystem, using the default {@code CommandHandle} of this sender to process responses.
     * If no default handle has been set, responses will be ignored.
     * <p>
     * This method returns immediately, without waiting for the command to be processed
     * by the messaging service or the target subsystem. Synchronous command execution can
     * be done by calling {@link CommandTask#get get()} or {@link CommandTask#getResult() getResult()}
     * on the {@link CommandTask} returned by this method.
     * 
     * @param timeout Timeout. Cannot be {@code null}.
     *                Zero duration means the command will never time out.
     *                Negative duration means its absolute value will be used and the custom timeout
     *                suggested by the target subsystem through an ACK will be ignored.
     * @param command Command in {@code subsystem/path} format.
     * @param args Command arguments.
     * @return CommandTask created to process the command. Can be used to wait for the response or cancel the command processing.
     */
    public CommandTask execute(Duration timeout, String command, Object... args) {
        return execute(defaultCommandHandle, timeout, command, args);
    }
    
    /**
     * Sends a command to a remote subsystem with the default timeout.
     * If no default timeout has been set for this sender, the timeout is 10 seconds,
     * unless the target subsystem returns a custom timeout with ACK.
     * <p>
     * This method returns immediately, without waiting for the command to be processed
     * by the messaging service or the target subsystem. Synchronous command execution can
     * be done by calling {@link CommandTask#get get()} or {@link CommandTask#getResult() getResult()}
     * on the {@link CommandTask} returned by this method.
     * 
     * @param handle Handle for processing the command responses.
     *               Cannot be {@code null}. If no processing is required, use {@code CommandHandle.NONE}.
     * @param command Command in subsystem/path format.
     * @param args Command arguments.
     * @return CommandTask created to process the command. Can be used to wait for the response or cancel the command processing.
     */
    public CommandTask execute(CommandHandle handle, String command, Object... args) {
        return execute(handle, defaultTimeout, command, args);
    }
    
    /**
     * Sends a command to a remote subsystem with the default timeout and the default callback handle.
     * The default timeout is 10 seconds, unless the target subsystem returns a custom timeout with ACK.
     * If no default handle has been set, responses will be ignored.
     * <p>
     * This method returns immediately, without waiting for the command to be processed
     * by the messaging service or the target subsystem. Synchronous command execution can
     * be done by calling {@link CommandTask#get get()} or {@link CommandTask#getResult() getResult()}
     * on the {@link CommandTask} returned by this method.
     * 
     * @param command Command in subsystem/path format.
     * @param args Command arguments.
     * @return CommandTask created to process the command. Can be used to wait for the response or cancel the command processing.
     */
    public CommandTask execute(String command, Object... args) {
        return execute(defaultCommandHandle, defaultTimeout, command, args);
    }
    
}
